@@ -6,7 +6,8 @@ import hashlib
 import tempfile
 import inspect
 from fsspec import AbstractFileSystem, filesystem
-from fsspec.core import MMapCache
+from fsspec.spec import AbstractBufferedFile
+from fsspec.core import MMapCache, BaseCache
 
 logger = logging.getLogger("fsspec")
 
@@ -40,6 +41,7 @@ class CachingFileSystem(AbstractFileSystem):
         check_files=False,
         expiry_time=604800,
         target_options=None,
+        **kwargs
     ):
         """
 
@@ -66,6 +68,9 @@ class CachingFileSystem(AbstractFileSystem):
         target_options: dict or None
             Passed to the instantiation of the FS, if fs is None.
         """
+        if self._cached:
+            return
+        super().__init__(**kwargs)
         if cache_storage == "TMP":
             storage = [tempfile.mkdtemp()]
         else:
@@ -74,21 +79,28 @@ class CachingFileSystem(AbstractFileSystem):
             else:
                 storage = cache_storage
         os.makedirs(storage[-1], exist_ok=True)
-        self.protocol = target_protocol
         self.storage = storage
         self.kwargs = target_options or {}
         self.cache_check = cache_check
         self.check_files = check_files
         self.expiry = expiry_time
+        self._target_protocol = target_protocol
+        self.target_protocol = (
+            target_protocol
+            if isinstance(target_protocol, str)
+            else target_protocol.protocol
+        )
         self.load_cache()
-        self.fs = filesystem(target_protocol, **self.kwargs)
-        super().__init__(**self.kwargs)
+        if isinstance(target_protocol, AbstractFileSystem):
+            self.fs = target_protocol
+        else:
+            self.fs = filesystem(target_protocol, **self.kwargs)
 
     def __reduce_ex__(self, *_):
         return (
             self.__class__,
             (
-                self.protocol,
+                self._target_protocol,
                 self.storage,
                 self.cache_check,
                 self.check_files,
@@ -126,18 +138,22 @@ class CachingFileSystem(AbstractFileSystem):
                     if cache[k]["blocks"] is True:
                         c["blocks"] = True
                     else:
-                        c["blocks"] = c["blocks"].union(cache[k]["blocks"])
+                        c["blocks"] = set(c["blocks"]).union(cache[k]["blocks"])
+
+            # Files can be added to cache after it was written once
+            for k, c in cache.items():
+                if k not in cached_files:
+                    cached_files[k] = c
         else:
             cached_files = cache
         cache = {k: v.copy() for k, v in cached_files.items()}
         for c in cache.values():
             if isinstance(c["blocks"], set):
                 c["blocks"] = list(c["blocks"])
-        with open(fn + ".temp", "wb") as f:
+        fn2 = tempfile.mktemp()
+        with open(fn2, "wb") as f:
             pickle.dump(cache, f)
-        if os.path.exists(fn):
-            os.remove(fn)
-        os.rename(fn + ".temp", fn)
+        os.replace(fn2, fn)
 
     def _check_cache(self):
         """Reload caches if time elapsed or any disappeared"""
@@ -152,6 +168,9 @@ class CachingFileSystem(AbstractFileSystem):
     def _check_file(self, path):
         """Is path in cache and still valid"""
         self._check_cache()
+        if not path.startswith(self.target_protocol):
+            store_path = self.target_protocol + "://" + path
+            path = self.fs._strip_protocol(store_path)
         for storage, cache in zip(self.storage, self.cached_files):
             if path not in cache:
                 continue
@@ -167,7 +186,15 @@ class CachingFileSystem(AbstractFileSystem):
                 return detail, fn
         return False, None
 
-    def _open(self, path, mode="rb", **kwargs):
+    def _open(
+        self,
+        path,
+        mode="rb",
+        block_size=None,
+        autocommit=True,
+        cache_options=None,
+        **kwargs
+    ):
         """Wrap the target _open
 
         If the whole file exists in the cache, just open it locally and
@@ -181,11 +208,20 @@ class CachingFileSystem(AbstractFileSystem):
         ``close_and_update`` to save the state of the blocks.
         """
         path = self._strip_protocol(path)
-        if not path.startswith(self.protocol):
-            path = self.protocol + "://" + path
+
+        if not path.startswith(self.target_protocol):
+            store_path = self.target_protocol + "://" + path
+            path = self.fs._strip_protocol(store_path)
         if mode != "rb":
-            return self.fs._open(path, mode=mode, **kwargs)
-        detail, fn = self._check_file(path)
+            return self.fs._open(
+                path,
+                mode=mode,
+                block_size=block_size,
+                autocommit=autocommit,
+                cache_options=cache_options,
+                **kwargs
+            )
+        detail, fn = self._check_file(store_path)
         if detail:
             # file is in cache
             hash, blocks = detail["fn"], detail["blocks"]
@@ -205,13 +241,19 @@ class CachingFileSystem(AbstractFileSystem):
                 "time": time.time(),
                 "uid": self.fs.ukey(path),
             }
-            self.cached_files[-1][path] = detail
+            self.cached_files[-1][store_path] = detail
             logger.debug("Creating local sparse file for %s" % path)
-        kwargs["cache_type"] = "none"
-        kwargs["mode"] = mode
 
         # call target filesystems open
-        f = self.fs._open(path, **kwargs)
+        f = self.fs._open(
+            path,
+            mode=mode,
+            block_size=block_size,
+            autocommit=autocommit,
+            cache_options=cache_options,
+            cache_type=None,
+            **kwargs
+        )
         if "blocksize" in detail:
             if detail["blocksize"] != f.blocksize:
                 raise ValueError(
@@ -224,15 +266,16 @@ class CachingFileSystem(AbstractFileSystem):
         f.cache = MMapCache(f.blocksize, f._fetch_range, f.size, fn, blocks)
         close = f.close
         f.close = lambda: self.close_and_update(f, close)
+        self.save_cache()
         return f
 
     def close_and_update(self, f, close):
         """Called when a file is closing, so store the set of blocks"""
-        if f.path.startswith(self.protocol):
-            path = f.path
-        else:
-            path = self.protocol + "://" + f.path
-        c = self.cached_files[-1][path]
+        path = self._strip_protocol(f.path)
+
+        if not path.startswith(self.target_protocol):
+            store_path = self.target_protocol + "://" + path
+        c = self.cached_files[-1][store_path]
         if c["blocks"] is not True and len(["blocks"]) * f.blocksize >= f.size:
             c["blocks"] = True
         self.save_cache()
@@ -255,10 +298,14 @@ class CachingFileSystem(AbstractFileSystem):
             "head",
             "_check_file",
             "_check_cache",
+            "glob",
         ]:
             # all the methods defined in this class. Note `open` here, since
             # it calls `_open`, but is actually in superclass
             return lambda *args, **kw: getattr(type(self), item)(self, *args, **kw)
+        if item in ["_strip_protocol"]:
+            # class methods
+            return lambda *args, **kw: getattr(type(self), item)(*args, **kw)
         if item == "__class__":
             return type(self)
         d = object.__getattribute__(self, "__dict__")
@@ -301,11 +348,13 @@ class WholeFileCacheFileSystem(CachingFileSystem):
 
     def _open(self, path, mode="rb", **kwargs):
         path = self._strip_protocol(path)
-        if not path.startswith(self.protocol):
-            path = self.protocol + "://" + path
+
+        if not path.startswith(self.target_protocol):
+            store_path = self.target_protocol + "://" + path
+            path = self.fs._strip_protocol(store_path)
         if mode != "rb":
             return self.fs._open(path, mode=mode, **kwargs)
-        detail, fn = self._check_file(path)
+        detail, fn = self._check_file(store_path)
         if detail:
             hash, blocks = detail["fn"], detail["blocks"]
             if blocks is True:
@@ -326,15 +375,18 @@ class WholeFileCacheFileSystem(CachingFileSystem):
                 "time": time.time(),
                 "uid": self.fs.ukey(path),
             }
-            self.cached_files[-1][path] = detail
+            self.cached_files[-1][store_path] = detail
             logger.debug("Copying %s to local cache" % path)
-        kwargs["cache_type"] = "none"
         kwargs["mode"] = mode
 
         # call target filesystems open
+        # TODO: why not just use fs.get ??
         f = self.fs._open(path, **kwargs)
         with open(fn, "wb") as f2:
-            if f.blocksize and f.size:
+            if isinstance(f, AbstractBufferedFile):
+                # want no type of caching if just downloading whole thing
+                f.cache = BaseCache(0, f.cache.fetcher, f.size)
+            if getattr(f, "blocksize", 0) and f.size:
                 # opportunity to parallelise here
                 data = True
                 while data:
@@ -345,3 +397,78 @@ class WholeFileCacheFileSystem(CachingFileSystem):
                 f2.write(f.read())
         self.save_cache()
         return self._open(path, mode)
+
+
+class SimpleCacheFileSystem(CachingFileSystem):
+    """Caches whole remote files on first access
+
+    This class is intended as a layer over any other file system, and
+    will make a local copy of each file accessed, so that all subsequent
+    reads are local. This implementation only copies whole files, and
+    does not keep any metadata about the download time or file details.
+    It is therefore safer to use in multi-threaded/concurrent situations.
+
+    """
+
+    protocol = "simplecache"
+
+    def __init__(
+        self, target_protocol=None, cache_storage="TMP", target_options=None, **kwargs
+    ):
+        for key in ["cache_check", "expiry_time", "check_files"]:
+            kwargs.pop(key, None)
+        super().__init__(
+            target_protocol=target_protocol,
+            target_options=target_options,
+            cache_storage=cache_storage,
+            cache_check=False,
+            expiry_time=False,
+            check_files=False,
+            **kwargs
+        )
+        for storage in self.storage:
+            if not os.path.exists(storage):
+                os.makedirs(storage, exist_ok=True)
+        self.cached_files = [{}]
+
+    def _check_file(self, path):
+        sha = hashlib.sha256(path.encode()).hexdigest()
+        for storage in self.storage:
+            fn = os.path.join(storage, sha)
+            if os.path.exists(fn):
+                return fn
+
+    def save_cache(self):
+        pass
+
+    def _open(self, path, mode="rb", **kwargs):
+        path = self._strip_protocol(path)
+
+        if not path.startswith(self.target_protocol):
+            store_path = self.target_protocol + "://" + path
+            path = self.fs._strip_protocol(store_path)
+        if mode != "rb":
+            return self.fs._open(path, mode=mode, **kwargs)
+        fn = self._check_file(path)
+        if fn:
+            return open(fn, "rb")
+
+        sha = hashlib.sha256(path.encode()).hexdigest()
+        fn = os.path.join(self.storage[-1], sha)
+        logger.debug("Copying %s to local cache" % path)
+        kwargs["mode"] = mode
+
+        with self.fs._open(path, **kwargs) as f, open(fn, "wb") as f2:
+            if isinstance(f, AbstractBufferedFile):
+                # want no type of caching if just downloading whole thing
+                f.cache = BaseCache(0, f.cache.fetcher, f.size)
+            if getattr(f, "blocksize", 0) and f.size:
+                # opportunity to parallelise here
+                data = True
+                while data:
+                    data = f.read(f.blocksize)
+                    f2.write(data)
+            else:
+                # this only applies to HTTP, should instead use streaming
+                f2.write(f.read())
+        return self._open(path, mode)
