@@ -6,6 +6,7 @@ from urllib.parse import urlparse
 from fsspec import AbstractFileSystem
 from fsspec.spec import AbstractBufferedFile
 from fsspec.utils import tokenize, DEFAULT_BLOCK_SIZE
+from ..caching import AllBytes
 
 # https://stackoverflow.com/a/15926317/3821154
 ex = re.compile(r"""<a\s+(?:[^>]*?\s+)?href=(["'])(.*?)\1""")
@@ -62,6 +63,8 @@ class HTTPFileSystem(AbstractFileSystem):
         """
         return path
 
+    # TODO: override get
+
     def ls(self, url, detail=True):
         # ignoring URL-encoded arguments
         r = self.session.get(url, **self.kwargs)
@@ -104,7 +107,7 @@ class HTTPFileSystem(AbstractFileSystem):
             return list(sorted(out))
 
     def cat(self, url):
-        r = requests.get(url, **self.kwargs)
+        r = self.session.get(url, **self.kwargs)
         r.raise_for_status()
         return r.content
 
@@ -122,12 +125,20 @@ class HTTPFileSystem(AbstractFileSystem):
         except requests.HTTPError:
             return False
 
-    def _open(self, url, mode="rb", block_size=None, **kwargs):
+    def _open(
+        self,
+        path,
+        mode="rb",
+        block_size=None,
+        autocommit=None,  # XXX: This differs from the base class.
+        cache_options=None,
+        **kwargs
+    ):
         """Make a file-like object
 
         Parameters
         ----------
-        url: str
+        path: str
             Full URL with protocol
         mode: string
             must be "rb"
@@ -141,13 +152,20 @@ class HTTPFileSystem(AbstractFileSystem):
             raise NotImplementedError
         block_size = block_size if block_size is not None else self.block_size
         kw = self.kwargs.copy()
-        kw.update(kwargs)
-        kw.pop("autocommit", None)
+        kw.update(kwargs)  # this does nothing?
         if block_size:
-            return HTTPFile(self, url, self.session, block_size, **kw)
+            return HTTPFile(
+                self,
+                path,
+                self.session,
+                block_size,
+                mode=mode,
+                cache_options=cache_options,
+                **kw
+            )
         else:
             kw["stream"] = True
-            r = self.session.get(url, **kw)
+            r = self.session.get(path, **kw)
             r.raise_for_status()
             r.raw.decode_content = True
             return r.raw
@@ -170,7 +188,7 @@ class HTTPFileSystem(AbstractFileSystem):
         for policy in ["head", "get"]:
             try:
                 size = file_size(url, self.session, policy, **self.kwargs)
-                if size is not None:
+                if size:
                     break
             except Exception:
                 pass
@@ -178,7 +196,7 @@ class HTTPFileSystem(AbstractFileSystem):
             # get failed, so conclude URL does not exist
             if size is False:
                 raise FileNotFoundError(url)
-        return {"name": url, "size": size, "type": "file"}
+        return {"name": url, "size": size or None, "type": "file"}
 
 
 class HTTPFile(AbstractBufferedFile):
@@ -214,6 +232,7 @@ class HTTPFile(AbstractBufferedFile):
         block_size=None,
         mode="rb",
         cache_type="bytes",
+        cache_options=None,
         size=None,
         **kwargs
     ):
@@ -229,6 +248,7 @@ class HTTPFile(AbstractBufferedFile):
             mode=mode,
             block_size=block_size,
             cache_type=cache_type,
+            cache_options=cache_options,
             **kwargs
         )
         self.cache.size = self.size or self.blocksize
@@ -340,13 +360,3 @@ def file_size(url, session=None, size_policy="head", **kwargs):
         return int(r.headers["Content-Length"])
     elif "Content-Range" in r.headers:
         return int(r.headers["Content-Range"].split("/")[1])
-
-
-class AllBytes(object):
-    """Cache entire contents of a remote URL"""
-
-    def __init__(self, data):
-        self.data = data
-
-    def _fetch(self, start, end):
-        return self.data[start:end]
