@@ -12,6 +12,13 @@ default = "file"
 known_implementations = {
     "file": {"class": "fsspec.implementations.local.LocalFileSystem"},
     "memory": {"class": "fsspec.implementations.memory.MemoryFileSystem"},
+    "dropbox": {
+        "class": "dropboxdrivefs.DropboxDriveFileSystem",
+        "err": (
+            'DropboxFileSystem requires "dropboxdrivefs",'
+            '"requests" and "dropbox" to be installed'
+        ),
+    },
     "http": {
         "class": "fsspec.implementations.http.HTTPFileSystem",
         "err": 'HTTPFileSystem requires "requests" to be installed',
@@ -58,10 +65,15 @@ known_implementations = {
     "cached": {"class": "fsspec.implementations.cached.CachingFileSystem"},
     "blockcache": {"class": "fsspec.implementations.cached.CachingFileSystem"},
     "filecache": {"class": "fsspec.implementations.cached.WholeFileCacheFileSystem"},
+    "simplecache": {"class": "fsspec.implementations.cached.SimpleCacheFileSystem"},
     "dask": {
         "class": "fsspec.implementations.dask.DaskWorkerFileSystem",
         "err": "Install dask distributed to access worker file system",
     },
+    "github": {
+        "class": "fsspec.implementations.github.GithubFileSystem",
+        "err": "Install the requests package to use the github FS",
+    },
 }
 
 minversions = {"s3fs": LooseVersion("0.3.0"), "gcsfs": LooseVersion("0.3.0")}
@@ -86,27 +98,7 @@ def get_filesystem_class(protocol):
         if protocol not in known_implementations:
             raise ValueError("Protocol not known: %s" % protocol)
         bit = known_implementations[protocol]
-        mod, name = bit["class"].rsplit(".", 1)
-        minversion = minversions.get(mod, None)
-        err = None
-        try:
-            mod = importlib.import_module(mod)
-        except ImportError:
-            err = ImportError(bit["err"])
-
-        except Exception as e:
-            err = e
-        if err is not None:
-            raise RuntimeError(str(err))
-
-        if minversion:
-            version = getattr(mod, "__version__", None)
-            if version and LooseVersion(version) < minversion:
-                raise RuntimeError(
-                    "'{}={}' is installed, but version '{}' or "
-                    "higher is required".format(mod.__name__, version, minversion)
-                )
-        registry[protocol] = getattr(mod, name)
+        registry[protocol] = _import_class(bit["class"])
     cls = registry[protocol]
     if getattr(cls, "protocol", None) in ("abstract", None):
         cls.protocol = protocol
@@ -114,6 +106,22 @@ def get_filesystem_class(protocol):
     return cls
 
 
+def _import_class(cls, minv=None):
+    mod, name = cls.rsplit(".", 1)
+    minv = minv or minversions
+    minversion = minv.get(mod, None)
+
+    mod = importlib.import_module(mod)
+    if minversion:
+        version = getattr(mod, "__version__", None)
+        if version and LooseVersion(version) < minversion:
+            raise RuntimeError(
+                "'{}={}' is installed, but version '{}' or "
+                "higher is required".format(mod.__name__, version, minversion)
+            )
+    return getattr(mod, name)
+
+
 def filesystem(protocol, **storage_options):
     """Instantiate filesystems for given protocol and arguments
 
