@@ -1,3 +1,4 @@
+import warnings
 from hashlib import md5
 import io
 import os
@@ -5,24 +6,61 @@ import logging
 
 from .transaction import Transaction
 from .utils import read_block, tokenize, stringify_path
+from .dircache import DirCache
 
 logger = logging.getLogger("fsspec")
 
-# alternative names for some methods, which get patched to new instances
-# (alias, original)
-aliases = [
-    ("makedir", "mkdir"),
-    ("mkdirs", "makedirs"),
-    ("listdir", "ls"),
-    ("cp", "copy"),
-    ("move", "mv"),
-    ("stat", "info"),
-    ("disk_usage", "du"),
-    ("rename", "mv"),
-    ("delete", "rm"),
-    ("upload", "put"),
-    ("download", "get"),
-]
+
+def make_instance(cls, args, kwargs):
+    return cls(*args, **kwargs)
+
+
+class _Cached(type):
+    """
+    Metaclass for caching file system instances.
+
+    Notes
+    -----
+    Instances are cached according to
+
+    * The values of the class attributes listed in `_extra_tokenize_attributes`
+    * The arguments passed to ``__init__``.
+
+    This creates an additional reference to the filesystem, which prevents the
+    filesystem from being garbage collected when all *user* references go away.
+    A call to the :meth:`AbstractFileSystem.clear_instance_cache` must *also*
+    be made for a filesystem instance to be garbage collected.
+    """
+
+    cachable = True
+    _extra_tokenize_attributes = ()
+
+    def __init__(cls, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        # Note: we intentionally create a reference here, to avoid garbage
+        # collecting instances when all other references are gone. To really
+        # delete a FileSystem, the cache must be cleared.
+        cls._cache = {}
+
+    def __call__(cls, *args, **kwargs):
+        extra_tokens = tuple(
+            getattr(cls, attr, None) for attr in cls._extra_tokenize_attributes
+        )
+        token = tokenize(cls, *args, *extra_tokens, **kwargs)
+        skip = kwargs.pop("skip_instance_cache", False)
+        if not skip and cls.cachable and token in cls._cache:
+            return cls._cache[token]
+        else:
+            obj = super().__call__(*args, **kwargs)
+            # Setting _fs_token here causes some static linters to complain.
+            obj._fs_token_ = token
+            obj.storage_args = args
+            obj.storage_options = kwargs
+
+            if cls.cachable and not skip:
+                cls._cache[token] = obj
+            return obj
+
 
 try:  # optionally derive from pyarrow's FileSystem, if available
     import pyarrow as pa
@@ -32,7 +70,7 @@ except ImportError:
     up = object
 
 
-class AbstractFileSystem(up):
+class AbstractFileSystem(up, metaclass=_Cached):
     """
     An abstract super-class for pythonic file-systems
 
@@ -40,8 +78,6 @@ class AbstractFileSystem(up):
     from here.
     """
 
-    _singleton = [None]  # will contain the newest instance
-    _cache = {}
     cachable = True  # this class can be cached, instances reused
     _cached = False
     blocksize = 2 ** 22
@@ -49,29 +85,8 @@ class AbstractFileSystem(up):
     protocol = "abstract"
     root_marker = ""  # For some FSs, may require leading '/' or other character
 
-    def __new__(cls, *args, **storage_options):
-        """
-        Will reuse existing instance if:
-        - cls.cachable is True and
-        - storage_options does not include do_cache=False
-        - the token (a hash of args and kwargs by default) exists in the cache
-
-        The instance will skip init if instance.cached = True.
-        """
-
-        # TODO: defer to a class-specific tokeniser?
-        do_cache = storage_options.pop("do_cache", True)
-        token = tokenize(cls, args, storage_options)
-        if cls.cachable and token in cls._cache and do_cache:
-            # check for cached instance
-            return cls._cache[token]
-        self = object.__new__(cls)
-        self._fs_token = token
-        if self.cachable:
-            # store for caching - can hold memory
-            cls._cache[token] = self
-        self.storage_options = storage_options
-        return self
+    #: Extra *class attributes* that should be considered when hashing.
+    _extra_tokenize_attributes = ()
 
     def __init__(self, *args, **storage_options):
         """Create and configure file-system instance
@@ -84,26 +99,36 @@ class AbstractFileSystem(up):
 
         Subclasses should call this method.
 
-        Magic kwargs that affect functionality here:
-        add_docs: if True, will append docstrings from this spec to the
-            specific implementation
-        add_aliases: if True, will add method aliases
+        Parameters
+        ----------
+        use_listings_cache, listings_expiry_time, max_paths:
+            passed to ``DirCache``, if the implementation supports
+            directory listing caching. Pass use_listings_cache=False
+            to disable such caching.
+        skip_instance_cache: bool
+            If this is a cachable implementation, pass True here to force
+            creating a new instance even if a matching instance exists, and prevent
+            storing this instance.
         """
         if self._cached:
             # reusing instance, don't change
             return
         self._cached = True
         self._intrans = False
-        self._transaction = Transaction(self)
-        self._singleton[0] = self
-        self.dircache = {}
-        if storage_options.pop("add_docs", True):
-            self._mangle_docstrings()
-        if storage_options.pop("add_aliases", True):
-            for new, old in aliases:
-                if not hasattr(self, new):
-                    # don't apply alias if attribute exists already
-                    setattr(self, new, getattr(self, old))
+        self._transaction = None
+        self.dircache = DirCache(**storage_options)
+
+        if storage_options.pop("add_docs", None):
+            warnings.warn("add_docs is no longer supported.", FutureWarning)
+
+        if storage_options.pop("add_aliases", None):
+            warnings.warn("add_aliases has been removed.", FutureWarning)
+        # This is set in _Cached
+        self._fs_token_ = None
+
+    @property
+    def _fs_token(self):
+        return self._fs_token_
 
     def __dask_tokenize__(self):
         return self._fs_token
@@ -112,40 +137,7 @@ class AbstractFileSystem(up):
         return int(self._fs_token, 16)
 
     def __eq__(self, other):
-        return self._fs_token == other._fs_token
-
-    @classmethod
-    def clear_instance_cache(cls, remove_singleton=True):
-        """Remove any instances stored in class attributes"""
-        cls._cache.clear()
-        if remove_singleton:
-            cls._singleton = [None]
-
-    def _mangle_docstrings(self):
-        """Add AbstractFileSystem docstrings to subclass methods
-
-        Disable by including ``add_docs=False`` to init kwargs.
-        """
-        for method in dir(self.__class__):
-            if method.startswith("_"):
-                continue
-            if self.__class__ is not AbstractFileSystem:
-                m = getattr(self.__class__, method)
-                n = getattr(AbstractFileSystem, method, None).__doc__
-                if not callable(m) or not n or n in (m.__doc__ or ""):
-                    # ignore if a) not a method, b) no superclass doc
-                    # c) already includes docstring
-                    continue
-                try:
-                    if m.__doc__:
-                        m.__doc__ += (
-                            "\n Upstream docstring: \n"
-                            + getattr(AbstractFileSystem, method).__doc__
-                        )
-                    else:
-                        m.__doc__ = getattr(AbstractFileSystem, method).__doc__
-                except AttributeError:
-                    pass
+        return isinstance(other, type(self)) and self._fs_token == other._fs_token
 
     @classmethod
     def _strip_protocol(cls, path):
@@ -183,10 +175,10 @@ class AbstractFileSystem(up):
 
         If no instance has been created, then create one with defaults
         """
-        if not cls._singleton[0]:
+        if not len(cls._cache):
             return cls()
         else:
-            return cls._singleton[0]
+            return list(cls._cache.values())[-1]
 
     @property
     def transaction(self):
@@ -195,16 +187,20 @@ class AbstractFileSystem(up):
         Requires the file class to implement `.commit()` and `.discard()`
         for the normal and exception cases.
         """
+        if self._transaction is None:
+            self._transaction = Transaction(self)
         return self._transaction
 
     def start_transaction(self):
         """Begin write transaction for deferring files, non-context version"""
         self._intrans = True
+        self._transaction = Transaction(self)
         return self.transaction
 
     def end_transaction(self):
         """Finish write transaction, non-context version"""
         self.transaction.complete()
+        self._transaction = None
 
     def invalidate_cache(self, path=None):
         """
@@ -256,7 +252,7 @@ class AbstractFileSystem(up):
         """Remove a directory, if empty"""
         pass  # not necessary to implement, may not have directories
 
-    def ls(self, path, **kwargs):
+    def ls(self, path, detail=True, **kwargs):
         """List objects at path.
 
         This should include subdirectories and files at that location. The
@@ -303,14 +299,18 @@ class AbstractFileSystem(up):
         but contains nothing), None if not in cache.
         """
         parent = self._parent(path)
-        if path in self.dircache:
+        try:
             return self.dircache[path]
-        elif parent in self.dircache:
+        except KeyError:
+            pass
+        try:
             files = [f for f in self.dircache[parent] if f["name"] == path]
             if len(files) == 0:
                 # parent dir was listed but did not contain this file
                 raise FileNotFoundError(path)
             return files
+        except KeyError:
+            pass
 
     def walk(self, path, maxdepth=None, **kwargs):
         """ Return all files belows path
@@ -331,38 +331,43 @@ class AbstractFileSystem(up):
         kwargs: passed to ``ls``
         """
         path = self._strip_protocol(path)
-        full_dirs = []
-        dirs = []
-        files = []
+        full_dirs = {}
+        dirs = {}
+        files = {}
 
+        detail = kwargs.pop("detail", False)
         try:
-            listing = self.ls(path, True, **kwargs)
+            listing = self.ls(path, detail=True, **kwargs)
         except (FileNotFoundError, IOError):
             return [], [], []
 
         for info in listing:
             # each info name must be at least [path]/part , but here
             # we check also for names like [path]/part/
-            name = info["name"].rstrip("/")
-            if info["type"] == "directory" and name != path:
+            pathname = info["name"].rstrip("/")
+            name = pathname.rsplit("/", 1)[-1]
+            if info["type"] == "directory" and pathname != path:
                 # do not include "self" path
-                full_dirs.append(name)
-                dirs.append(name.rsplit("/", 1)[-1])
-            elif name == path:
+                full_dirs[pathname] = info
+                dirs[name] = info
+            elif pathname == path:
                 # file-like with same name as give path
-                files.append("")
+                files[""] = info
             else:
-                files.append(name.rsplit("/", 1)[-1])
-        yield path, dirs, files
+                files[name] = info
+
+        if detail:
+            yield path, dirs, files
+        else:
+            yield path, list(dirs), list(files)
+
+        if maxdepth is not None:
+            maxdepth -= 1
+            if maxdepth < 1:
+                return
 
         for d in full_dirs:
-            if maxdepth is None or maxdepth > 1:
-                for res in self.walk(
-                    d,
-                    maxdepth=(maxdepth - 1) if maxdepth is not None else None,
-                    **kwargs
-                ):
-                    yield res
+            yield from self.walk(d, maxdepth=maxdepth, detail=detail, **kwargs)
 
     def find(self, path, maxdepth=None, withdirs=False, **kwargs):
         """List all files below path.
@@ -371,6 +376,7 @@ class AbstractFileSystem(up):
 
         Parameters
         ----------
+        path : str
         maxdepth: int or None
             If not None, the maximum number of levels to descend
         withdirs: bool
@@ -379,18 +385,22 @@ class AbstractFileSystem(up):
         kwargs are passed to ``ls``.
         """
         # TODO: allow equivalent of -name parameter
-        out = set()
-        for path, dirs, files in self.walk(path, maxdepth, **kwargs):
+        path = self._strip_protocol(path)
+        out = dict()
+        detail = kwargs.pop("detail", False)
+        for path, dirs, files in self.walk(path, maxdepth, detail=True, **kwargs):
             if withdirs:
-                files += dirs
-            for name in files:
-                if name and name not in out:
-                    out.add("/".join([path.rstrip("/"), name]) if path else name)
+                files.update(dirs)
+            out.update({info["name"]: info for name, info in files.items()})
         if self.isfile(path) and path not in out:
             # walk works on directories, but find should also return [path]
             # when path happens to be a file
-            out.add(path)
-        return sorted(out)
+            out[path] = {}
+        names = sorted(out)
+        if not detail:
+            return names
+        else:
+            return {name: out[name] for name in names}
 
     def du(self, path, total=True, maxdepth=None, **kwargs):
         """Space used by files within a path
@@ -431,21 +441,33 @@ class AbstractFileSystem(up):
         kwargs are passed to ``ls``.
         """
         import re
+        from glob import has_magic
 
         ends = path.endswith("/")
         path = self._strip_protocol(path)
         indstar = path.find("*") if path.find("*") >= 0 else len(path)
         indques = path.find("?") if path.find("?") >= 0 else len(path)
-        ind = min(indstar, indques)
-        if "*" not in path and "?" not in path:
+        indbrace = path.find("[") if path.find("[") >= 0 else len(path)
+
+        ind = min(indstar, indques, indbrace)
+
+        detail = kwargs.pop("detail", False)
+
+        if not has_magic(path):
             root = path
             depth = 1
             if ends:
                 path += "/*"
             elif self.exists(path):
-                return [path]
+                if not detail:
+                    return [path]
+                else:
+                    return {path: self.info(path)}
             else:
-                return []  # glob of non-existent returns empty
+                if not detail:
+                    return []  # glob of non-existent returns empty
+                else:
+                    return {}
         elif "/" in path[:ind]:
             ind2 = path[:ind].rindex("/")
             root = path[: ind2 + 1]
@@ -453,7 +475,8 @@ class AbstractFileSystem(up):
         else:
             root = ""
             depth = 20 if "**" in path else 1
-        allpaths = self.find(root, maxdepth=depth, withdirs=True, **kwargs)
+
+        allpaths = self.find(root, maxdepth=depth, withdirs=True, detail=True, **kwargs)
         pattern = (
             "^"
             + (
@@ -472,15 +495,23 @@ class AbstractFileSystem(up):
         pattern = re.sub("[*]{2}", "=PLACEHOLDER=", pattern)
         pattern = re.sub("[*]", "[^/]*", pattern)
         pattern = re.compile(pattern.replace("=PLACEHOLDER=", ".*"))
-        out = {p for p in allpaths if pattern.match(p.replace("//", "/").rstrip("/"))}
-        return list(sorted(out))
+        out = {
+            p: allpaths[p]
+            for p in sorted(allpaths)
+            if pattern.match(p.replace("//", "/").rstrip("/"))
+        }
+        if detail:
+            return out
+        else:
+            return list(out)
 
     def exists(self, path):
         """Is there a file at the given path"""
         try:
             self.info(path)
             return True
-        except:  # any exception allowed bar FileNotFoundError?
+        except:  # noqa: E722
+            # any exception allowed bar FileNotFoundError?
             return False
 
     def info(self, path, **kwargs):
@@ -538,14 +569,14 @@ class AbstractFileSystem(up):
         """Is this entry directory-like?"""
         try:
             return self.info(path)["type"] == "directory"
-        except FileNotFoundError:
+        except IOError:
             return False
 
     def isfile(self, path):
         """Is this entry file-like?"""
         try:
             return self.info(path)["type"] == "file"
-        except:
+        except:  # noqa: E722
             return False
 
     def cat(self, path):
@@ -553,7 +584,7 @@ class AbstractFileSystem(up):
         return self.open(path, "rb").read()
 
     def get(self, rpath, lpath, recursive=False, **kwargs):
-        """ Copy file to local
+        """Copy file to local.
 
         Possible extension: maybe should be able to copy to any file-system
         (streaming through local).
@@ -581,19 +612,28 @@ class AbstractFileSystem(up):
 
     def put(self, lpath, rpath, recursive=False, **kwargs):
         """ Upload file from local """
+        from .implementations.local import make_path_posix
+        import posixpath
+
         if recursive:
             lpaths = []
             for dirname, subdirlist, filelist in os.walk(lpath):
-                lpaths += [os.path.join(dirname, filename) for filename in filelist]
-            rootdir = os.path.basename(lpath.rstrip("/"))
+                lpaths += [
+                    make_path_posix(os.path.join(dirname, filename))
+                    for filename in filelist
+                ]
+            rootdir = make_path_posix(
+                os.path.basename(make_path_posix(lpath).rstrip("/"))
+            )
             if self.exists(rpath):
                 # copy lpath inside rpath directory
-                rpath2 = os.path.join(rpath, rootdir)
+                rpath2 = posixpath.join(rpath, rootdir)
             else:
                 # copy lpath as rpath directory
                 rpath2 = rpath
             rpaths = [
-                os.path.join(rpath2, path[len(lpath) :].lstrip("/")) for path in lpaths
+                posixpath.join(rpath2, path[len(lpath) :].lstrip("/"))
+                for path in lpaths
             ]
         else:
             lpaths = [lpath]
@@ -651,11 +691,11 @@ class AbstractFileSystem(up):
         for p in path:
             if recursive:
                 out = self.walk(p, maxdepth=maxdepth)
-                for pa, _, files in reversed(list(out)):
+                for pa_, _, files in reversed(list(out)):
                     for name in files:
-                        fn = "/".join([pa, name]) if pa else name
+                        fn = "/".join([pa_, name]) if pa_ else name
                         self.rm(fn)
-                    self.rmdir(pa)
+                    self.rmdir(pa_)
             else:
                 self._rm(p)
 
@@ -667,11 +707,27 @@ class AbstractFileSystem(up):
         else:
             return cls.root_marker
 
-    def _open(self, path, mode="rb", block_size=None, autocommit=True, **kwargs):
+    def _open(
+        self,
+        path,
+        mode="rb",
+        block_size=None,
+        autocommit=True,
+        cache_options=None,
+        **kwargs
+    ):
         """Return raw bytes-mode file-like from the file-system"""
-        return AbstractBufferedFile(self, path, mode, block_size, autocommit)
+        return AbstractBufferedFile(
+            self,
+            path,
+            mode,
+            block_size,
+            autocommit,
+            cache_options=cache_options,
+            **kwargs
+        )
 
-    def open(self, path, mode="rb", block_size=None, **kwargs):
+    def open(self, path, mode="rb", block_size=None, cache_options=None, **kwargs):
         """
         Return a file-like object from the filesystem
 
@@ -686,6 +742,8 @@ class AbstractFileSystem(up):
             See builtin ``open()``
         block_size: int
             Some indication of buffering - this is a value in bytes
+        cache_options : dict, optional
+            Extra arguments to pass through to the cache.
         encoding, errors, newline: passed on to TextIOWrapper for text mode
         """
         import io
@@ -705,7 +763,12 @@ class AbstractFileSystem(up):
         else:
             ac = kwargs.pop("autocommit", not self._intrans)
             f = self._open(
-                path, mode=mode, block_size=block_size, autocommit=ac, **kwargs
+                path,
+                mode=mode,
+                block_size=block_size,
+                autocommit=ac,
+                cache_options=cache_options,
+                **kwargs
             )
             if not ac:
                 self.transaction.files.append(f)
@@ -777,15 +840,61 @@ class AbstractFileSystem(up):
                 length = size - offset
             return read_block(f, offset, length, delimiter)
 
-    def __getstate__(self):
-        """ Instance should be pickleable """
-        d = self.__dict__.copy()
-        d.pop("dircache")
-        return d
+    def __reduce__(self):
+        return make_instance, (type(self), self.storage_args, self.storage_options)
 
-    def __setstate__(self, state):
-        self.__dict__.update(state)
-        self.dircache = {}
+    def to_json(self):
+        """
+        JSON representation of this filesystem instance
+
+        Returns
+        -------
+        str: JSON structure with keys cls (the python location of this class),
+            protocol (text name of this class's protocol, first one in case of
+            multiple), args (positional args, usually empty), and all other
+            kwargs as their own keys.
+        """
+        import json
+
+        cls = type(self)
+        cls = ".".join((cls.__module__, cls.__name__))
+        proto = (
+            self.protocol[0]
+            if isinstance(self.protocol, (tuple, list))
+            else self.protocol
+        )
+        return json.dumps(
+            dict(
+                **{"cls": cls, "protocol": proto, "args": self.storage_args},
+                **self.storage_options
+            )
+        )
+
+    @staticmethod
+    def from_json(blob):
+        """
+        Recreate a filesystem instance from JSON representation
+
+        See ``.to_json()`` for the expected structure of the input
+
+        Parameters
+        ----------
+        blob: str
+
+        Returns
+        -------
+        file system instance, not necessarily of this particular class.
+        """
+        from .registry import _import_class, get_filesystem_class
+        import json
+
+        dic = json.loads(blob)
+        protocol = dic.pop("protocol")
+        try:
+            cls = _import_class(dic.pop("cls"))
+        except (ImportError, ValueError, RuntimeError, KeyError):
+            cls = get_filesystem_class(protocol)
+        return cls(*dic.pop("args", ()), **dic)
 
     def _get_pyarrow_filesystem(self):
         """
@@ -806,9 +915,66 @@ class AbstractFileSystem(up):
 
     @classmethod
     def clear_instance_cache(cls):
-        """Remove cached instances from the class cache"""
+        """
+        Clear the cache of filesystem instances.
+
+        Notes
+        -----
+        Unless overridden by setting the ``cachable`` class attribute to False,
+        the filesystem class stores a reference to newly created instances. This
+        prevents Python's normal rules around garbage collection from working,
+        since the instances refcount will not drop to zero until
+        ``clear_instance_cache`` is called.
+        """
         cls._cache.clear()
 
+    # ------------------------------------------------------------------------
+    # Aliases
+
+    def makedir(self, path, create_parents=True, **kwargs):
+        """Alias of :ref:`FilesystemSpec.mkdir`."""
+        return self.mkdir(path, create_parents=create_parents, **kwargs)
+
+    def mkdirs(self, path, exist_ok=False):
+        """Alias of :ref:`FilesystemSpec.makedirs`."""
+        return self.makedirs(path, exist_ok=exist_ok)
+
+    def listdir(self, path, detail=True, **kwargs):
+        """Alias of :ref:`FilesystemSpec.ls`."""
+        return self.ls(path, detail=detail, **kwargs)
+
+    def cp(self, path1, path2, **kwargs):
+        """Alias of :ref:`FilesystemSpec.copy`."""
+        return self.copy(path1, path2, **kwargs)
+
+    def move(self, path1, path2, **kwargs):
+        """Alias of :ref:`FilesystemSpec.mv`."""
+        return self.mv(path1, path2, **kwargs)
+
+    def stat(self, path, **kwargs):
+        """Alias of :ref:`FilesystemSpec.info`."""
+        return self.info(path, **kwargs)
+
+    def disk_usage(self, path, total=True, maxdepth=None, **kwargs):
+        """Alias of :ref:`FilesystemSpec.du`."""
+        return self.du(path, total=total, maxdepth=maxdepth, **kwargs)
+
+    def rename(self, path1, path2, **kwargs):
+        """Alias of :ref:`FilesystemSpec.mv`."""
+        return self.mv(path1, path2, **kwargs)
+
+    def delete(self, path, recursive=False, maxdepth=None):
+        """Alias of :ref:`FilesystemSpec.rm`."""
+        return self.rm(path, recursive=recursive, maxdepth=maxdepth)
+
+    def upload(self, lpath, rpath, recursive=False, **kwargs):
+        """Alias of :ref:`FilesystemSpec.put`."""
+        return self.put(lpath, rpath, recursive=recursive, **kwargs)
+
+    def download(self, rpath, lpath, recursive=False, **kwargs):
+        """Alias of :ref:`FilesystemSpec.get`."""
+        return self.get(rpath, lpath, recursive=recursive, **kwargs)
+
 
 class AbstractBufferedFile(io.IOBase):
     """Convenient class to derive from to provide buffering
@@ -828,7 +994,8 @@ class AbstractBufferedFile(io.IOBase):
         mode="rb",
         block_size="default",
         autocommit=True,
-        cache_type="bytes",
+        cache_type="readahead",
+        cache_options=None,
         **kwargs
     ):
         """
@@ -847,9 +1014,11 @@ class AbstractBufferedFile(io.IOBase):
         autocommit: bool
             Whether to write to final destination; may only impact what
             happens when file is being closed.
-        cache_type: str
-            Caching policy in read mode, one of 'none', 'bytes', 'mmap', see
-            the definitions in ``core``.
+        cache_type: {"readahead", "none", "mmap", "bytes"}, default "readahead"
+            Caching policy in read mode. See the definitions in ``core``.
+        cache_options : dict
+            Additional options passed to the constructor for the cache specified
+            by `cache_type`.
         kwargs:
             Gets stored as self.kwargs
         """
@@ -866,8 +1035,20 @@ class AbstractBufferedFile(io.IOBase):
         self.end = None
         self.start = None
         self.closed = False
-        self.trim = kwargs.pop("trim", True)
+
+        if cache_options is None:
+            cache_options = {}
+
+        if "trim" in kwargs:
+            warnings.warn(
+                "Passing 'trim' to control the cache behavior has been deprecated. "
+                "Specify it within the 'cache_options' argument instead.",
+                FutureWarning,
+            )
+            cache_options["trim"] = kwargs.pop("trim")
+
         self.kwargs = kwargs
+
         if mode not in {"ab", "rb", "wb"}:
             raise NotImplementedError("File mode not supported")
         if mode == "rb":
@@ -875,7 +1056,7 @@ class AbstractBufferedFile(io.IOBase):
                 self.details = fs.info(path)
             self.size = self.details["size"]
             self.cache = caches[cache_type](
-                self.blocksize, self._fetch_range, self.size, trim=self.trim
+                self.blocksize, self._fetch_range, self.size, **cache_options
             )
         else:
             self.buffer = io.BytesIO()
@@ -931,7 +1112,7 @@ class AbstractBufferedFile(io.IOBase):
         """
         loc = int(loc)
         if not self.mode == "rb":
-            raise ValueError("Seek only available in read mode")
+            raise OSError("Seek only available in read mode")
         if whence == 0:
             nloc = loc
         elif whence == 1:
@@ -1043,6 +1224,9 @@ class AbstractBufferedFile(io.IOBase):
         if self.closed:
             raise ValueError("I/O operation on closed file.")
         logger.debug("%s read: %i - %i" % (self, self.loc, self.loc + length))
+        if length == 0:
+            # don't even bother calling fetch
+            return b""
         out = self.cache._fetch(self.loc, self.loc + length)
         self.loc += len(out)
         return out
